import unittest
import datetime

class Test_Utc(unittest.TestCase):
    def _makeOne(self):
        from ..iso8601 import Utc
        return Utc()

    def test_utcoffset(self):
        from ..iso8601 import ZERO
        inst = self._makeOne()
        result = inst.utcoffset(None)
        self.assertEqual(result, ZERO)

    def test_tzname(self):
        inst = self._makeOne()
        result = inst.tzname(None)
        self.assertEqual(result, "UTC")

    def test_dst(self):
        from ..iso8601 import ZERO
        inst = self._makeOne()
        result = inst.dst(None)
        self.assertEqual(result, ZERO)

    def test_picklability(self):
        from ..iso8601 import ZERO
        from ..compat import loads, dumps, HIGHEST_PROTOCOL
        inst = self._makeOne()
        for protocol in range(HIGHEST_PROTOCOL + 1):
            inst2 = loads(dumps(inst, protocol))
            self.assertEqual(inst2.utcoffset(None), ZERO)
            self.assertEqual(inst2.tzname(None), 'UTC')
            self.assertEqual(inst2.dst(None), ZERO)

class Test_FixedOffset(unittest.TestCase):
    def _makeOne(self):
        from ..iso8601 import FixedOffset
        return FixedOffset(1, 30, 'oneandahalf')

    def test_utcoffset(self):
        inst = self._makeOne()
        result = inst.utcoffset(None)
        self.assertEqual(result, datetime.timedelta(hours=1, minutes=30))

    def test_tzname(self):
        inst = self._makeOne()
        result = inst.tzname(None)
        self.assertEqual(result, 'oneandahalf')

    def test_dst(self):
        from ..iso8601 import ZERO
        inst = self._makeOne()
        result = inst.dst(None)
        self.assertEqual(result, ZERO)

    def test_picklability(self):
        from ..iso8601 import ZERO
        from ..compat import loads, dumps, HIGHEST_PROTOCOL
        inst = self._makeOne()
        for protocol in range(HIGHEST_PROTOCOL + 1):
            inst2 = loads(dumps(inst, protocol))
            self.assertEqual(inst2.utcoffset(None),
                            datetime.timedelta(hours=1, minutes=30))
            self.assertEqual(inst2.tzname(None), 'oneandahalf')
            self.assertEqual(inst2.dst(None), ZERO)

    def test___repr__(self):
        inst = self._makeOne()
        result = inst.__repr__()
        self.assertEqual(result, "<FixedOffset 'oneandahalf' datetime.timedelta(0, 5400)>")

class Test_parse_timezone(unittest.TestCase):
    def _callFUT(self, tzstring, **kw):
        # mimic old parse_timezone() by returning a FixedOffset
        from datetime import tzinfo
        from ..iso8601 import (parse_date, FixedOffset)
        if tzstring is None:
            tzstring = ''
        dt = parse_date("2006-10-11T00:14:33{0}".format(tzstring), **kw)
        return dt.tzinfo

    def test_default_Z(self):
        from ..iso8601 import UTC
        result = self._callFUT('Z')
        self.assertEqual(result, UTC)

    def test_Z_with_default_timezone(self):
        from ..iso8601 import UTC, FixedOffset
        tz = FixedOffset(1, 0, 'myname')
        result = self._callFUT('Z', default_timezone=tz)
        self.assertEqual(result, UTC)

    def test_default_None(self):
        from ..iso8601 import UTC
        result = self._callFUT(None)
        self.assertEqual(result, UTC)

    def test_None_with_default_timezone(self):
        from ..iso8601 import FixedOffset
        tz = FixedOffset(1, 0, 'myname')
        result = self._callFUT(None, default_timezone=tz)
        self.assertEqual(result, tz)

    def test_positive(self):
        tzstring = "+01:00"
        result = self._callFUT(tzstring)
        self.assertEqual(result.utcoffset(None),
                         datetime.timedelta(hours=1, minutes=0))

    def test_positive_without_colon(self):
        tzstring = "+0100"
        result = self._callFUT(tzstring)
        self.assertEqual(result.utcoffset(None),
                         datetime.timedelta(hours=1, minutes=0))

    def test_positive_without_minutes(self):
        tzstring = "+01"
        result = self._callFUT(tzstring)
        self.assertEqual(result.utcoffset(None),
                         datetime.timedelta(hours=1, minutes=0))

    def test_negative(self):
        tzstring = "-01:00"
        result = self._callFUT(tzstring)
        self.assertEqual(result.utcoffset(None),
                         datetime.timedelta(hours=-1, minutes=0))

    def test_negative_without_colon(self):
        tzstring = "-0100"
        result = self._callFUT(tzstring)
        self.assertEqual(result.utcoffset(None),
                         datetime.timedelta(hours=-1, minutes=0))

    def test_negative_without_minutes(self):
        tzstring = "-01"
        result = self._callFUT(tzstring)
        self.assertEqual(result.utcoffset(None),
                         datetime.timedelta(hours=-1, minutes=0))

class Test_parse_date(unittest.TestCase):
    def _callFUT(self, datestring):
        from ..iso8601 import parse_date
        return parse_date(datestring)

    def test_notastring(self):
        from ..iso8601 import ParseError
        self.assertRaises(ParseError, self._callFUT, None)

    def test_cantparse(self):
        from ..iso8601 import ParseError
        self.assertRaises(ParseError, self._callFUT, 'garbage')

    def test_outfrange(self):
        from ..iso8601 import ParseError
        self.assertRaises(ParseError, self._callFUT, '2013-05-32')

    def test_normal(self):
        from ..iso8601 import UTC
        result = self._callFUT("2007-01-25T12:00:00Z")
        self.assertEqual(result,
                         datetime.datetime(2007, 1, 25, 12, 0, tzinfo=UTC))

    def test_fraction(self):
        from ..iso8601 import UTC
        result = self._callFUT("2007-01-25T12:00:00.123Z")
        self.assertEqual(result,
                         datetime.datetime(2007, 1, 25, 12, 0, 0, 123000,
                                           tzinfo=UTC))

    def test_no_seconds(self):
        from ..iso8601 import UTC
        result = self._callFUT("2007-01-25T12:00Z")
        self.assertEqual(result,
                         datetime.datetime(2007, 1, 25, 12, 0, 0, 0,
                                           tzinfo=UTC))

    def test_no_minutes(self):
        from ..iso8601 import UTC
        result = self._callFUT("2007-01-25T12Z")
        self.assertEqual(result,
                         datetime.datetime(2007, 1, 25, 12, 0, 0, 0,
                                           tzinfo=UTC))

    def test_no_hours(self):
        from ..iso8601 import UTC
        result = self._callFUT("2007-01-25")
        self.assertEqual(result,
                         datetime.datetime(2007, 1, 25, 0, 0, 0, 0,
                                           tzinfo=UTC))

    def test_slash_separated_raises_ParseError(self):
        from ..iso8601 import ParseError
        self.assertRaises(ParseError, self._callFUT, "2007/01/25")




from __future__ import unicode_literals
import logging

from . import _SchemaMeta
from . import SchemaNode
from . import Mapping


logger = logging.getLogger(__name__)


def get_root_class(bases, super_cls):
    """Get root class which has super_cls in base classes

    """
    bases_set = set(bases)
    root_cls = None
    while root_cls is None and bases_set:
        next_bases_set = set()
        for base_cls in bases_set:
            if AbstractSchema in base_cls.__bases__:
                root_cls = base_cls
                break
            next_bases_set |= set(base_cls.__bases__)
        bases_set = next_bases_set
    return root_cls


class _AbstractMeta(_SchemaMeta):
    def __init__(cls, name, bases, clsattrs):
        def super_init():
            return super(_AbstractMeta, cls).__init__(name, bases, clsattrs)
        # AbstractSchema class, skip
        if bases == (SchemaNode, ):
            return super_init()
        # this class inherts Abstract as parent
        if AbstractSchema in bases:
            if '__mapper_args__' not in clsattrs:
                raise TypeError('__mapper_args__ should be defined')
            if 'polymorphic_on' not in clsattrs['__mapper_args__']:
                raise TypeError('__mapper_args__ should has polymorphic_on')
            cls.__polymorphic_mapping__ = {}
        else:
            # find the root class, for example:
            #     + AbstractSchema
            #         + root
            #             + foo
            #                 + bar
            # so the root class will be `root`
            root_cls = get_root_class(bases, AbstractSchema)
            # register this class to root class
            polymorphic_on = root_cls.__mapper_args__['polymorphic_on']
            polymorphic_id = clsattrs[polymorphic_on]
            if polymorphic_id in root_cls.__polymorphic_mapping__:
                raise KeyError('%s already exists' % polymorphic_id)
            root_cls.__polymorphic_mapping__[polymorphic_id] = cls

            logger.info(
                'Register class %s to root class %s as %s',
                cls, root_cls, polymorphic_id,
            )
        return super_init()


class Abstract(Mapping):

    def _get_subnode(self, node, data):
        polymorphic_on = node.__mapper_args__['polymorphic_on']
        polymorphic_id = data[polymorphic_on]
        root_cls = get_root_class((node.__class__, ), AbstractSchema)
        cls = root_cls.__polymorphic_mapping__[polymorphic_id]
        subnode = cls()
        return subnode

    def serialize(self, node, appstruct, not_abstract=False):
        if not_abstract:
            return super(Abstract, self).serialize(node, appstruct)
        subnode = self._get_subnode(node, appstruct)
        return subnode.typ.serialize(subnode, appstruct, not_abstract=True)

    def deserialize(self, node, cstruct, not_abstract=False):
        if not_abstract:
            return super(Abstract, self).deserialize(node, cstruct)
        subnode = self._get_subnode(node, cstruct)
        return subnode.typ.deserialize(subnode, cstruct, not_abstract=True)


class AbstractSchema(SchemaNode):
    __metaclass__ = _AbstractMeta
    schema_type = Abstract



# -*- coding:utf-8 -*-
import unittest
import datetime


class TestPolymorphism(unittest.TestCase):
    def _makeSchema(self, name='schema'):
        import colander
        from colander.polymorphism import AbstractSchema

        class Payload(AbstractSchema):
            cls_type = colander.SchemaNode(colander.String())
            __mapper_args__ = {
                'polymorphic_on': 'cls_type',
            }

        class Log(Payload):
            cls_type = 'log'
            message = colander.SchemaNode(colander.String())
            severity = colander.SchemaNode(colander.String(), validators=[
                colander.OneOf((
                    'debug',
                    'info',
                    'warning',
                    'error',
                ))
            ])

        class DatetimeLog(Log):
            cls_type = 'datetime-log'
            created_at = colander.SchemaNode(colander.DateTime())

        class Event(Payload):
            cls_type = 'event'
            tag = colander.SchemaNode(colander.String())
            data = colander.SchemaNode(colander.String())

        class MessageSchema(colander.MappingSchema):
            payload = Payload()

        schema = MessageSchema()
        return schema

    def test_serialize(self):
        from colander.iso8601 import Utc
        now = datetime.datetime.utcnow().replace(tzinfo=Utc())
        schema = self._makeSchema()

        result = schema.serialize({
            'payload': {
                'cls_type': 'log',
                'message': 'hello',
                'severity': 'info',
            }
        })
        self.assertEqual(result, {
            'payload': {
                'cls_type': 'log',
                'message': 'hello',
                'severity': 'info',
            }
        })

        result = schema.serialize({
            'payload': {
                'cls_type': 'datetime-log',
                'message': 'hello',
                'severity': 'info',
                'created_at': now,
            }
        })
        self.assertEqual(result, {
            'payload': {
                'cls_type': 'datetime-log',
                'message': 'hello',
                'severity': 'info',
                'created_at': now.isoformat(),
            }
        })

        result = schema.serialize({
            'payload': {
                'cls_type': 'event',
                'tag': 'foobar',
                'data': 'hi',
            }
        })
        self.assertEqual(result, {
            'payload': {
                'cls_type': 'event',
                'tag': 'foobar',
                'data': 'hi',
            }
        })

    def test_deserialize(self):
        from colander.iso8601 import Utc
        now = datetime.datetime.utcnow().replace(tzinfo=Utc())
        schema = self._makeSchema()
        result = schema.deserialize({
            'payload': {
                'cls_type': 'log',
                'message': 'hello',
                'severity': 'info',
            }
        })
        self.assertEqual(result, {
            'payload': {
                'cls_type': 'log',
                'message': 'hello',
                'severity': 'info',
            }
        })

        result = schema.deserialize({
            'payload': {
                'cls_type': 'datetime-log',
                'message': 'hello',
                'severity': 'info',
                'created_at': now.isoformat(),
            }
        })
        self.assertEqual(result, {
            'payload': {
                'cls_type': 'datetime-log',
                'message': 'hello',
                'severity': 'info',
                'created_at': now,
            }
        })

        result = schema.deserialize({
            'payload': {
                'cls_type': 'event',
                'tag': 'foobar',
                'data': 'hi',
            }
        })
        self.assertEqual(result, {
            'payload': {
                'cls_type': 'event',
                'tag': 'foobar',
                'data': 'hi',
            }
        })

    def test_not_exist_type(self):
        schema = self._makeSchema()
        with self.assertRaises(KeyError):
            schema.serialize({
                'payload': {
                    'cls_type': 'does_not_exist',
                    'message': 'hello',
                    'severity': 'info',
                }
            })
        with self.assertRaises(KeyError):
            schema.deserialize({
                'payload': {
                    'cls_type': 'does_not_exist',
                    'message': 'hello',
                    'severity': 'info',
                }
            })

    def test_duplicate_polymorphic_id(self):
        import colander
        from colander.polymorphism import AbstractSchema

        class Payload(AbstractSchema):
            cls_type = colander.SchemaNode(colander.String())
            __mapper_args__ = {
                'polymorphic_on': 'cls_type',
            }

        class Log(Payload):
            cls_type = 'log'

        with self.assertRaises(KeyError):
            class AnotherLog(Log):
                cls_type = 'log'



##############################################################################
#
# Copyright (c) 2011 Agendaless Consulting and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE
#
##############################################################################

import os

from setuptools import setup
from setuptools import find_packages

here = os.path.abspath(os.path.dirname(__file__))

def read(fname):
    with open(fname) as fp:
        return fp.read()

try:
    README = read(os.path.join(here, 'README.txt'))
    CHANGES = read(os.path.join(here, 'CHANGES.txt'))
except:
    README = ''
    CHANGES = ''

requires = ['translationstring', 'iso8601']

testing_extras = ['nose', 'coverage']
docs_extras = ['Sphinx']

# This is a branch for Balanced
setup(name='ba-colander',
      version='1.0b1',
      description=('A simple schema-based serialization and deserialization '
                   'library'),
      long_description=README + '\n\n' +  CHANGES,
      classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        ],
      keywords='serialize deserialize validate schema validation',
      author="Agendaless Consulting",
      author_email="pylons-discuss@googlegroups.com",
      url="http://docs.pylonsproject.org/projects/colander/en/latest/",
      license="BSD-derived (http://www.repoze.org/LICENSE.txt)",
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires = requires,
      test_suite="colander",
      extras_require = {
          'testing':testing_extras,
          'docs':docs_extras,
          },
      )

